require 'sinatra/base'
require 'sinatra/json'
require 'sinatra/namespace'

require 'yaml'
require 'themoviedb'

module Librarix
  class Application < Sinatra::Application
    namespace '/api/v1' do
      not_found do
        {error: :not_found}.to_json
      end

      helpers do
        def base_url
          @base_url ||= "#{request.env['rack.url_scheme']}://#{request.env['HTTP_HOST']}"
        end

        def json_params
          begin
            JSON.parse(request.body.read)
          rescue
            halt 400, { message:'Invalid JSON' }.to_json
          end
        end
      end

      before do
        content_type 'application/json'
      end

      namespace '/genres' do
        get '' do
          Librarix::Models::Genre.fetch.to_json
        end
      end

      namespace '/movies' do
        get '' do
          Librarix::Filter.new(params).movies.to_json
        end

        post '' do
          id = json_params['tmdb_id'].to_i
          movie = Tmdb::Movie.detail(id)

          if movie['status_code'] != 34 && !Librarix::Redis::Movie.new(id).added?
            Librarix::Redis::Movie.new(id).add.to_json
          end
        end

        get '/:id' do |id|
          Librarix::Models::Movie.find(id: id).to_json
        end

        get '/:id/fetch' do |id|
          Librarix::Models::Movie.fetch(85).to_json
        end

        patch '/:id/view' do |id|
          Librarix::Models::Movie.find(id: id).view!
          {success: true}.to_json
        end

        patch '/:id/download' do |id|
          Librarix::Models::Movie.find(id: id).download!
          {success: true}.to_json
        end

        delete '/:id' do |id|
          Librarix::Models::Movie.find(id: id).destroy
          {success: true}.to_json
        end
      end
    end
  end
end
